//
//  NMAAuthClient.h
//  NMAAuthLib0
//
//  Created by iam on 7/10/15.
//  Copyright (c) 2015 NetIQ. All rights reserved.
//
//  OAuth2Client
//
//  Created by Ullrich Schäfer on 27.08.10.
//
//  Copyright 2010 nxtbgthng. All rights reserved.
//
//  Licenced under the new BSD-licence.
//  See README.md in this repository for
//  the full licence.
//

#import <Foundation/Foundation.h>

#import "NMAAuthClientDelegate.h"
#import "NMAAuthConnectionDelegate.h"

extern NSString * const NMAAuthClientConnectionContextTokenRequest;
extern NSString * const NMAAuthClientConnectionContextTokenRefresh;

@class NMAAuthConnection, NMAAuthToken;

/*!
 * The OAuth 2.0 client
 * Only supports code & Password flow at the moment
 *
 * - oauth2 draft 10 http://tools.ietf.org/html/draft-ietf-oauth-v2-10
 * - not thread save
 */

@interface NMAAuthClient : NSObject <NMAAuthConnectionDelegate> {

@protected
    BOOL authenticating;
    BOOL persistent;
    
    NSString    *clientId;
    NSString    *clientSecret;
    
    NSSet       *desiredScope;
    NSString    *userAgent;
    NSString    *assertion;
    NSString    *keyChainGroup;
    NSString    *keyChainAccessGroup;
    
    // server information
    NSURL        *authorizeURL;
    NSURL        *tokenURL;
    NSString     *tokenType;
    
    // token exchange
    NMAAuthConnection    *authConnection;
    NMAAuthToken    *accessToken;
    NSMutableArray    *waitingConnections; //for connections that are waiting for successful authorisation
    NSInteger        refreshConnectionDidRetryCount;
    
    // delegates
    NSObject<NMAAuthClientDelegate>*    __unsafe_unretained delegate;    // assigned
}

@property (nonatomic, readonly, getter = isAuthenticating) BOOL authenticating;

@property (nonatomic, copy, readonly) NSString *clientId;
@property (nonatomic, copy, readonly) NSString *clientSecret;
@property (nonatomic, copy, readonly) NSString *tokenType;
@property (nonatomic, strong, readwrite) NSDictionary *additionalAuthenticationParameters;
@property (nonatomic, strong, readwrite) NSDictionary *customHeaderFields;

@property (nonatomic, copy) NSSet *desiredScope;
@property (nonatomic, copy) NSString *tokenRequestHTTPMethod; // defaults to POST
@property (nonatomic, copy) NSString *userAgent;
@property (nonatomic, copy) NSString *acceptType; // defaults to application/json

@property (nonatomic, strong) NMAAuthToken    *accessToken;
@property (nonatomic, unsafe_unretained) NSObject<NMAAuthClientDelegate>*    delegate;

@property (nonatomic, readonly) NMAAuthConnection *authConnection;

/*!
 * If set to NO, the access token is not stored any keychain, will be removed if it was.
 * Defaults to YES
 */
@property (nonatomic, assign, readwrite, getter=isPersistent) BOOL persistent;

/*!
 * Initializes the Client
 */
- (instancetype)initWithClientID:(NSString *)clientId
                    clientSecret:(NSString *)clientSecret
                    authorizeURL:(NSURL *)authorizeURL
                        tokenURL:(NSURL *)tokenURL
                        delegate:(NSObject<NMAAuthClientDelegate> *)delegate;

- (instancetype)initWithClientID:(NSString *)clientId
                    clientSecret:(NSString *)clientSecret
                    authorizeURL:(NSURL *)authorizeURL
                        tokenURL:(NSURL *)tokenURL
                     accessToken:(NMAAuthToken *)accessToken
                   keyChainGroup:(NSString *)keyChainGroup
             keyChainAccessGroup:(NSString *)keyChainAccessGroup
                      persistent:(BOOL)shouldPersist
                        delegate:(NSObject<NMAAuthClientDelegate> *)delegate;

- (instancetype)initWithClientID:(NSString *)clientId
                    clientSecret:(NSString *)clientSecret
                    authorizeURL:(NSURL *)authorizeURL
                        tokenURL:(NSURL *)tokenURL
                     accessToken:(NMAAuthToken *)accessToken
                       tokenType:(NSString *)tokenType
                   keyChainGroup:(NSString *)keyChainGroup
             keyChainAccessGroup:(NSString *)keyChainAccessGroup
                      persistent:(BOOL)shouldPersist
                        delegate:(NSObject<NMAAuthClientDelegate> *)delegate;

- (BOOL)openRedirectURL:(NSURL *)URL;
- (BOOL)openRedirectURL:(NSURL *)URL error: (NSError**) error;


#pragma mark Authorisation Methods

/*---------------------------------*
 * Use ONE of the following flows: *
 *---------------------------------*/

/*!
 * Authenticate usind a web URL (Web Server Flow)
 * returns the URL to be opened to get access grant
 */
- (NSURL *)authorizationURLWithRedirectURL:(NSURL *)redirectURL;

/*!
 * Authenticate with username & password (User Credentials Flow)
 */
- (void)authenticateWithClientCredentials;
- (void)authenticateWithUsername:(NSString *)username password:(NSString *)password;

/*!
 * Authenticate with assertion (Assertion Flow)
 */
- (void)authenticateWithAssertionType:(NSURL *)assertionType assertion:(NSString *)assertion;


#pragma mark Public

- (void)requestAccess;
- (void)requestAccessAndRetryConnection:(NMAAuthConnection *)retryConnection;

- (NSHTTPURLResponse *) refreshAccessTokenSynchronousWithError:(NSError * __autoreleasing *) error;
- (void)refreshAccessToken;
- (void)refreshAccessTokenAndRetryConnection:(NMAAuthConnection *)retryConnection;

- (void)removeConnectionFromWaitingQueue:(NMAAuthConnection *)aConnection;

@end
